package com.fnic.security.jaas;


import java.io.IOException;
//import java.security.Principal;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
//import java.util.Set;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.ChoiceCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.TextOutputCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.message.callback.CallerPrincipalCallback;
import javax.security.auth.message.callback.GroupPrincipalCallback;
import javax.security.auth.message.callback.PasswordValidationCallback;

import org.apache.log4j.Logger;

import com.sun.enterprise.security.SecurityContext;





/**
 * <p>Implementation of the JAAS <code>CallbackHandler</code> interface,
 * used to negotiate delivery of the username and credentials that were
 * specified to our constructor.  No interaction with the user is required
 * (or possible).</p>
 *
 * <p>This <code>CallbackHandler</code> will <code>pre-digest</code> the supplied
 * password, if required by the <code>&lt;Realm&gt;</code> element in 
 * <code>server.xml</code>.</p>
 * <p>At present, <code>FNISCallbackHandler</code> knows how to handle callbacks of
 * type <code>javax.security.auth.callback.NameCallback</code> and
 * <code>javax.security.auth.callback.PasswordCallback</code>.</p>
 * Based on Apache code provided by: Craig R. McClanahan, Andrew R. Jaquith
 *
 * Added support for <code>additional callbacks</code> needed for the FidelitySolutions Web app
 *
 * @author Gerry Cata
 * Extended this class to handle the Choice & TextOutput callbacks
 * @version $Id: FNISCallbackHandler.java 1.0 2012-03-08 00:08:58Z gwcata $
 */

public class FNISCallbackHandler implements CallbackHandler {


    //------------------------------------------------------ Instance Variables

    /**
     * The execution user subject.
     */
	Subject clientSubject = null;

    /**
     * The password to be authenticated with.
     */
    protected String password = null;


    /**
     * The username to be authenticated with.
     */
    protected String username = null;

    /**
     * Realm name.
     */
    protected String realmName;

    /**
     * The authentication method to be used. If null, assume BASIC/FORM.
     */
    protected String authMethod;
    
    /**
     * The authentication options that are set. 
     */
    @SuppressWarnings("rawtypes")
	protected Map authOptions;
    
    // ------------------------------------------------------------ Constructor

	public static Logger logger = Logger.getLogger(FNISCallbackHandler.class);
    /**
     * Construct a callback handler configured with the specified values.
     *
     * @param realm Our associated FNISRealm instance
     * @param uname Username to be authenticated with
     * @param pwd Password to be authenticated with
     */
    @SuppressWarnings("rawtypes")
	public FNISCallbackHandler(Subject client, String realm, String uname, String pwd, Map options) {
        super();
        clientSubject = client;
        realmName = realm;
        username = uname;
        password = pwd;
        authOptions = options;
    }

    /**
     * Construct an alternate callback handler configured with the specified values.
     *
     * @param realm Our associated FNISRealm instance
     * @param uname Username to be authenticated with
     * @param pwd Password to be authenticated with
     */
	public FNISCallbackHandler(String realm, String uname, String pwd) {
        super();
        realmName = realm;
        username = uname;
        password = pwd;
    }
    // --------------------------------------------------------- Public Methods


    public void handle(Callback callbacks[])
        throws IOException, UnsupportedCallbackException {
		List<Callback> cbList = new ArrayList<Callback>();
		try { 
	        for (int i = 0; i < callbacks.length; i++) {
	            if (callbacks[i] instanceof NameCallback) {
	                logger.debug("FNISCallbackHandler.handle: received NameCallback - username [" + username + "]");
	                NameCallback ncb = (NameCallback) callbacks[i];
	                ncb.setName(username);
	            	//cbList.add(ncb);
	            } else if (callbacks[i] instanceof PasswordCallback) {
	                final char[] passwordcontents;
	                if (password != null) {
	                    passwordcontents = password.toCharArray();
	                } else {
	                    passwordcontents = new char[0];
	                }
	                PasswordCallback pwdcb = (PasswordCallback) callbacks[i];
	                pwdcb.setPassword(passwordcontents);
	            	//cbList.add(pwdcb);
	            } else if (callbacks[i] instanceof PasswordValidationCallback) {
	            	PasswordValidationCallback pvc = (PasswordValidationCallback)callbacks[i];
	            	logger.debug("FNISCallbackHandler.handle: received PasswordValidationCallback");
	            	cbList.add(pvc);
	            } else if (callbacks[i] instanceof CallerPrincipalCallback) {
	            	CallerPrincipalCallback cpc = (CallerPrincipalCallback) callbacks[i];
	            	logger.debug("FNISCallbackHandler.handle: received CallerPrincipalCallback");
	            	cbList.add(cpc);
	            } else if (callbacks[i] instanceof GroupPrincipalCallback) {
	            	GroupPrincipalCallback gpcb = (GroupPrincipalCallback) callbacks[i];
	            	logger.debug("FNISCallbackHandler.handle: received GroupPrincipalCallback");
	            	cbList.add(gpcb);
	            } else if (callbacks[i] instanceof ChoiceCallback) {
					ChoiceCallback cCB = ((ChoiceCallback) callbacks[i]);
					if (cCB == null) {
						logger.debug("FNISCallbackHandler.handle: ChoiceCallback - choice callback list is null");
					} else {
						String [] choices = cCB.getChoices();
						for (int j = 0; j < choices.length; j++)
							logger.debug("FNISCallbackHandler.handle: ChoiceCallback - choice callback list = \n" + choices[j]);
					}
	            } else if (callbacks[i] instanceof TextOutputCallback) {
					TextOutputCallback toCB = ((TextOutputCallback) callbacks[i]);
					if (toCB != null) {
						if (toCB.getMessageType() == TextOutputCallback.ERROR)
							System.err.println(toCB.getMessage());
						else
							System.out.println(toCB.getMessage());
					} else {
						logger.debug("FNISCallbackHandler.handle: TextOutputCallback - text output callback list is null");
					}
	            } else {
	        		logger.debug("FNISCallbackHandler.handle: Undefined callback request received.  Callback Info: " + callbacks[i].getClass().getName());
	                throw new UnsupportedCallbackException(callbacks[i]);
	            }
	        }
	        if (cbList.size() > 0) {
		        Callback[] cbArray = new Callback[cbList.size()];
		        int i = 0;
		        for (Object cbElem : cbList) {
		        	cbArray[i++] = (Callback) cbElem;
		        }
		        processResults(cbArray);
	        }
		} catch (Exception cbe) {
    		logger.error("FNISCallbackHandler.handle: Exception encountered: " + cbe.getMessage() + "\n" + cbe.getStackTrace());
		}
    }
    
    /**
     * Clears out password state.
     */
    public void clearPassword() {
    	char[] pwd = password.toCharArray();
        if (pwd != null) {
            for (int i = 0; i < pwd.length; i++)
                pwd[i] = ' ';
            password = null;
        }
    }
    
    private void processResults(Callback[] callbacks) {
		Subject wrkSbj = new Subject();
		Set<Principal> principals = clientSubject.getPrincipals();
		if (principals != null && principals.size() == 1) {
			//process if there is only one principal
			for (Principal principal : principals) {
				wrkSbj.getPrincipals().add(principal);
			}
			wrkSbj.getPublicCredentials().addAll(clientSubject.getPublicCredentials());
			wrkSbj.getPrivateCredentials().addAll(clientSubject.getPrivateCredentials());
		}
		
		for (Callback callback : callbacks) {
			if (callback instanceof CallerPrincipalCallback) {
				logger.debug("FNISCallbackHandler.processResults: processing CallerPrincipalCallback");
				CallerPrincipalCallback cpc = (CallerPrincipalCallback) callback;
				wrkSbj.getPrincipals().addAll(cpc.getSubject().getPrincipals());
				wrkSbj.getPublicCredentials().addAll(cpc.getSubject().getPublicCredentials());
				wrkSbj.getPrivateCredentials().addAll(cpc.getSubject().getPrivateCredentials());
			} else if (callback instanceof GroupPrincipalCallback) {
				logger.debug("FNISCallbackHandler.processResults: processing GroupPrincipalCallback");
				GroupPrincipalCallback gpc = (GroupPrincipalCallback) callback;
				wrkSbj.getPrincipals().addAll(gpc.getSubject().getPrincipals());
				wrkSbj.getPublicCredentials().addAll(gpc.getSubject().getPublicCredentials());
				wrkSbj.getPrivateCredentials().addAll(gpc.getSubject().getPrivateCredentials());
			} else if (callback instanceof PasswordValidationCallback) {
				logger.debug("FNISCallbackHandler.processResults: processing PasswordValidationCallback");
				PasswordValidationCallback pvc = (PasswordValidationCallback) callback;
				wrkSbj.getPrincipals().addAll(pvc.getSubject().getPrincipals());
				wrkSbj.getPublicCredentials().addAll(pvc.getSubject().getPublicCredentials());
				wrkSbj.getPrivateCredentials().addAll(pvc.getSubject().getPrivateCredentials());
			} else if (callback instanceof NameCallback) {
				logger.debug("FNISCallbackHandler.processResults: processing NameCallback");
				NameCallback nCB = (NameCallback) callback;
				FNISPrincipal user = new FNISPrincipal(nCB.getName());
				wrkSbj.getPrincipals().add(user);
			} else if (callback instanceof PasswordCallback) {
				logger.debug("FNISCallbackHandler.processResults: processing PasswordCallback");
				PasswordCallback pwCB = (PasswordCallback) callback;
				pwCB.clearPassword();
			}
		}
		wrkSbj.setReadOnly();
		logger.debug("FNISCallbackHandler.processResults: Before creating new security context. Principals [" +
				wrkSbj.getPrincipals() + "]\nPrivate Credentials [" + wrkSbj.getPrivateCredentials() +
				"]\nPublic Credentials [" + wrkSbj.getPublicCredentials() + "] ");
		SecurityContext.setCurrent(new SecurityContext(wrkSbj));
  }
}
