package com.fnic.security.jaas;

import java.util.*;
import java.io.Serializable;
import java.security.Principal;
import customUtils.utils.*;
import static customUtils.constants.Constants.*;

	/**
	 * @author Gerry Cata
	 *
	 */
	public class FNISPrincipal implements Principal, Serializable {

		private static final long serialVersionUID = 1L;
		private static final String NAME = ": name: ";
		private static final String APPLICATIONS = "\nApplications: ";
		
		private final List<String> applications = new ArrayList<String>();
		private String name = EMPTY_STRING;

		public FNISPrincipal(String name){
			super();
			if (name == null)
				throw new NullPointerException("Illegal null input.");
			setName(name);
		}
		
		@Override
		public String toString() {
			return this.getClass().getName() + NAME + name + APPLICATIONS 
				+ applications.toString();
		}

		@Override
		public String getName() {
			return name;
		}
		
		public void setName(String name) {
			this.name = StringUtils.removeNullAndTrim(name);
		}

	    /**
	     * Compares the specified Object with this FNISPrincipal
	     * for equality.  Returns true if the given object is also a
	     * FNISPrincipaland the two FNISPrincipals
	     * have the same username.
	     * @param o Object to be compared for equality with this FNISPrincipal.
	     * @return true if the specified Object is equal equal to this * FNISPrincipal.
	*/
	public boolean equals(Object o){
		 if (o == null) 
			 return false; 
		 if (this == o) 
			 return true; 
		 if (! (o instanceof FNISPrincipal)) 
			 return false;
		 FNISPrincipal target = (FNISPrincipal) o;
		 if (this.getName().equals(target.getName())) 
			 return true; 
		 return false;
	 }
	/** * Return a hash code for this FNISPrincipal. * *
	* * @return a hash code for this FNISPrincipal. */
	public int hashCode() {
		return name.hashCode();
	}

	public List<String> getApplications() {
		return applications;
	}
}
