package com.fnic.security.jaas.oracle.glassfish;

import customUtils.utils.Logging;
import javax.security.auth.spi.LoginModule;
import javax.security.auth.Subject;
import javax.security.auth.login.*;
import javax.security.auth.callback.CallbackHandler;
import javax.servlet.http.*;
import java.util.Map;
import org.apache.log4j.Logger;

/**
 * The SessionLoginContext extends the JAAS LoginContext, so that,
 * when bound to an HttpSession, it will execute login(), and when
 * the session times out, it will execute logout(). One can bind
 * the SessionLoginContext to the session, by calling<br><pre>
 *      session.setAttribute("loginContext", myLoginContext);
 * </pre>
 *
 * Based on Apache code provided by: Paul Feuer and John Musser
 *
 * Customized as needed for the FidelitySolutions Web app
 *
 * @author Gerry Cata
 * @version$Id: FNICCLoginContext.java 1.0 2012-03-08 00:08:58Z gwcata $
 */

@SuppressWarnings("unused")
public class FNISLoginContext extends LoginContext implements HttpSessionBindingListener {
	private static Logger logger = Logging.getLogger(FNISLoginContext.class, "ServerSideUserAccounts", "FNISSAM");
    /**
     * Default constructor. See javax.security.auth.login.LoginContext
     * for details.
     */
    public FNISLoginContext(String name) throws LoginException {
        super(name);
		if (logger.isDebugEnabled()) {
			logger.debug("FNISLoginContext.defaultConstructor: After call to parent LoginContext.");
		}
    }

    /**
     * Default constructor. See javax.security.auth.login.LoginContext
     * for details.
     */
    public FNISLoginContext(String name, CallbackHandler callbackHandler) throws LoginException {
        super(name, callbackHandler);
		if (logger.isDebugEnabled()) {
			logger.debug("FNISLoginContext.customConstructor: After call to parent LoginContext.");
		}
    }

    /**
     * Default constructor. See javax.security.auth.login.LoginContext
     * for details.
     */
    public FNISLoginContext(String name, Subject subject) throws LoginException {
        super(name, subject);
    }

    /**
     * Default constructor. See javax.security.auth.login.LoginContext
     * for details.
     */
    public FNISLoginContext(String name, Subject subject, CallbackHandler callbackHandler) throws LoginException {
        super(name, subject, callbackHandler);
    }

    /**
     * Notifies the object that it is being bound to a
     * session and identifies the session.
     *
     * @param event the event that identifies the session
     */
    public void valueBound(HttpSessionBindingEvent event) {
		if (logger.isDebugEnabled()) {
			logger.debug("FNISLoginContext.valueBound: Received a valueBound event [" + event.getName() + "]");
		}
        try {
            login();
    		if (logger.isDebugEnabled()) {
    			logger.debug("FNISLoginContext.valueBound: After invoking login method");
    		}
        } catch (LoginException ex) {
            throw new java.lang.RuntimeException(ex.getMessage());
        }
    }

    /**
     * Notifies the object that it is being unbound from a
     * session and identifies the session.
     *
     * @param event the event that identifies the session
     */
    public void valueUnbound(HttpSessionBindingEvent event) {
		if (logger.isDebugEnabled()) {
			logger.debug("FNISLoginContext.valueBound: Received a valueUnbound event [" + event.getName() + "]");
		}
        try {
            logout();
    		if (logger.isDebugEnabled()) {
    			logger.debug("FNISLoginContext.valueBound: After invoking logout method");
    		}
        } catch (LoginException ex) {
            throw new java.lang.RuntimeException(ex.getMessage());
        }
    }
}

