package com.fnic.security.jaas.oracle.glassfish;

import customUtils.utils.Logging;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.message.AuthException;
import javax.security.auth.message.AuthStatus;
import javax.security.auth.message.MessageInfo;
import javax.security.auth.message.MessagePolicy;
import javax.security.auth.message.callback.CallerPrincipalCallback;
import javax.security.auth.message.callback.GroupPrincipalCallback;
import javax.security.auth.message.callback.PasswordValidationCallback;
import javax.security.auth.message.module.ServerAuthModule;
import javax.security.auth.login.LoginException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;

import com.fnic.security.jaas.FNISCallbackHandler;
import com.fnic.security.jaas.FNISPrincipal;



public class FNISSAM implements ServerAuthModule
{
	private static Logger logger = Logging.getLogger(FNISSAM.class, "ServerSideUserAccounts", "FNISSAM");
	public static final String OPT_DEBUG = "debug";

	@SuppressWarnings("rawtypes")
	protected static final Class[] supportedMessageTypes = { 
		HttpServletRequest.class, 
		HttpServletResponse.class };
	@SuppressWarnings("unused")
	private MessagePolicy requestPolicy;
	@SuppressWarnings("unused")
	private MessagePolicy responsePolicy;
	private CallbackHandler gfCBHandler;
	private Map<String,String> options;
	private String realmName = null;
	private ArrayList<String> defaultGroup = new ArrayList<String>();

	private boolean debug = false;
	private FNISCallbackHandler fnisCBH;
	private FNISLoginContext flctx = null;
	private String password = null;
	private String userName = null;

	@SuppressWarnings("unused")
	private String env = null;
	private static final String REALM_PROPERTY_NAME =
	          "realm.name";
	      private static final String GROUP_PROPERTY_NAME =
	          "group.name";
	      @SuppressWarnings("unused")
		private static final String BASIC = "Basic";
	      static final String AUTHORIZATION_HEADER =
	          "authorization";
	      static final String AUTHENTICATION_HEADER =
	          "WWW-Authenticate";

			private static final String AUTH_TYPE_INFO_KEY =
				"javax.servlet.http.authType";

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void initialize(MessagePolicy rqstPolicy, MessagePolicy rspPolicy, CallbackHandler cBH, Map opts)
		throws AuthException
	{
		requestPolicy = rqstPolicy;
		responsePolicy = rspPolicy;
		gfCBHandler = cBH;
		options = opts;
		if (options != null) {
			logger.info("FNISSAM.initialize: options map: " + options.get("debug"));
			debug = "true".equalsIgnoreCase((String)options.get("debug"));
			realmName = ((String)options.get(REALM_PROPERTY_NAME));
			if (options.containsKey(GROUP_PROPERTY_NAME))
				defaultGroup.add(options.get("group.name"));
		} else {
		  logger.info("FNISSAM.initialize: WARNING!!!! JAAS config options map is --NULL--");
		}
	}

	@SuppressWarnings("rawtypes")
	public Class[] getSupportedMessageTypes()
	{
		return supportedMessageTypes;
	}

	public AuthStatus validateRequest(MessageInfo msgInfo, Subject client, Subject server)
		throws AuthException
	{
		ArrayList<String> groups = new ArrayList<String>();
		AuthStatus status = null;
		boolean gotLTPA = false; boolean gotUser = false;
		HttpServletRequest request = (HttpServletRequest)msgInfo.getRequestMessage();
		HttpServletResponse response = null;
		response = (HttpServletResponse)msgInfo.getResponseMessage();
		userName = request.getParameter("j_username");
		password = request.getParameter("j_password");


		Cookie[] cookies = request.getCookies();
		if (cookies == null) {
			return AuthStatus.SUCCESS;
		}
		for (Cookie cookie : cookies) {
			if (cookie.getName().equals("LtpaToken"))
				gotLTPA = true;
			else if (cookie.getName().equals("FIRSTUserID"))
				gotUser = true;
		}
		if ((gotLTPA) && (gotUser)) {
			return AuthStatus.SUCCESS;
		} else {
			if ((userName == null) || (userName.equals(""))) {
				if (debug) 
					logger.debug("FNISSAM.validateRequest: login username or password entered.");
				return sendCredentialsRequest(msgInfo);
			}
		}
		if (! options.containsKey("AppName")) {
			String appName = request.getContextPath();
			options.put("AppName", appName.substring(1));
		}
						// Invoke the GlassFish Container Authentication Services
		try {			//	Invoke the FNIS Authentication Services
			if (debug)
				logger.debug("FNISSAM.validateRequest: Creating FNISCallbackHandler & FNISLoginContext");
			// The following is Needed since GF CBH does not include the Name,Password, Choice and TextOutput Callback Handlers
			fnisCBH = new FNISCallbackHandler(client, realmName, userName, password, options);
			flctx = new FNISLoginContext("AS400Login", fnisCBH);
			flctx.login();
			if (debug)
				logger.debug("FNISSAM.validateRequest: Completed FNISLoginContext login");
			if (response != null) {
				Cookie ltpa = new Cookie("LtpaToken", request.getRequestedSessionId());
				response.addCookie(ltpa);
			}
		} catch (LoginException flce) {
			logger.error("FNISSAM.validateRequest: Exception encountered processing FNISLoginContext. \n" + 
			flce.getMessage() + "\n" + flce.getStackTrace());
			status = AuthStatus.FAILURE;
			return status;
		}
		try {
			if (debug)
				logger.debug("FNISSAM.validateRequest: Preparing to invoke callbacks.");
			FNISPrincipal fnisPrincipal = new FNISPrincipal(userName);
			fnisPrincipal.getApplications().add(options.get("AppName"));
			client.getPrincipals().add(fnisPrincipal);
			PasswordCredential pwdCrdtl = new PasswordCredential(userName, password.toCharArray());
			client.getPrivateCredentials().add(pwdCrdtl);
			client.getPublicCredentials().add(fnisPrincipal);
	        groups.add("Fidelity_Users");
			GroupPrincipalCallback gpCB = new GroupPrincipalCallback(client, groups.toArray(new String[groups.size()]));
			if (debug)
				logger.debug("FNISSAM.validateRequest: Invoking GroupPrincipalCallback.");
			fnisCBH.handle(new Callback[] {gpCB} ); 
			CallerPrincipalCallback cpCB = new CallerPrincipalCallback(client, userName);
			if (debug)
				logger.debug("FNISSAM.validateRequest: Invoking CallerPrincipalCallback.");
			fnisCBH.handle(new Callback[] {cpCB} ); 
			PasswordValidationCallback pwdvCB = new PasswordValidationCallback(client, userName, password.toCharArray());
			if (debug)
				logger.debug("FNISSAM.validateRequest: Invoking PasswordValidationCallback.");
			fnisCBH.handle(new Callback[] {pwdvCB} ); 
			if (pwdvCB.getResult()) {
				try {
					setAuthenticationResult(userName, client, msgInfo);
				} catch (Exception e) {
					logger.error("FNISSAM.validateRequest: Exception encountered processing FNISLoginContext. \n");
					AuthException ae = new AuthException();
		              ae.initCause(e);
		              throw ae;
				} 
				status = AuthStatus.SUCCESS;
			}
		} catch (Exception cbe) {
			logger.error("FNISSAM.validateRequest: Exception encountered processing callbacks. \n" + 
			cbe.getMessage() + "\n" + cbe.getStackTrace());
			AuthException ae = new AuthException();
            ae.initCause(cbe);
            throw ae;
		}
		return status;
	}

	public AuthStatus secureResponse(MessageInfo msgInfo, Subject service)
		throws AuthException
	{
		return AuthStatus.SEND_SUCCESS;
	}
	
	private AuthStatus sendCredentialsRequest(MessageInfo msgInfo) {
		/*
		HttpServletResponse response = (HttpServletResponse) msgInfo.getResponseMessage();
		String header = BASIC + " realm=\"" + realmName + "\"";
	       response.setHeader(AUTHENTICATION_HEADER, header);
	       response.setStatus(
	               HttpServletResponse.SC_UNAUTHORIZED);		
	    */
	       return AuthStatus.SUCCESS;
	}

	// distinguish the caller principal
	// and assign default groups
	@SuppressWarnings("unchecked")
	private void setAuthenticationResult(String loginID,Subject subject, MessageInfo m) 
			throws IOException, UnsupportedCallbackException 
	{
		ArrayList<String> groups = new ArrayList<String>();
		gfCBHandler.handle(new Callback[]{ new CallerPrincipalCallback(subject, loginID)});
		if (loginID != null) {
			// add the default group if the property is set
            groups.addAll(defaultGroup);
            groups.add("Fidelity_Users");
			if (defaultGroup != null) {
				gfCBHandler.handle(new Callback[]{new GroupPrincipalCallback(subject, groups.toArray(new String[groups.size()]))});
			}
			m.getMap().put(AUTH_TYPE_INFO_KEY, "FNISRealm");
		}
	}

	public void cleanSubject(MessageInfo msgInfo, Subject subject) throws AuthException
	{
		if (subject != null)
			subject.getPrincipals().clear();
	}

}
